/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.couponrecord.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.v3.oas.annotations.tags.Tag;
import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.biz.modular.couponrecord.entity.BizCouponRecord;
import vip.xiaonuo.biz.modular.couponrecord.param.BizCouponRecordAddParam;
import vip.xiaonuo.biz.modular.couponrecord.param.BizCouponRecordEditParam;
import vip.xiaonuo.biz.modular.couponrecord.param.BizCouponRecordIdParam;
import vip.xiaonuo.biz.modular.couponrecord.param.BizCouponRecordPageParam;
import vip.xiaonuo.biz.modular.couponrecord.service.BizCouponRecordService;

import jakarta.annotation.Resource;
import jakarta.validation.Valid;
import jakarta.validation.constraints.NotEmpty;
import java.util.List;

/**
 * 优惠券记录控制器
 *
 * @author wulei
 * @date  2025/02/06 17:38
 */
@Tag(name = "优惠券记录控制器")
@RestController
@Validated
public class BizCouponRecordController {

    @Resource
    private BizCouponRecordService bizCouponRecordService;

    /**
     * 获取优惠券记录分页
     *
     * @author wulei
     * @date  2025/02/06 17:38
     */
    @Operation(summary = "获取优惠券记录分页")
    @SaCheckPermission("/biz/couponrecord/page")
    @GetMapping("/biz/couponrecord/page")
    public CommonResult<Page<BizCouponRecord>> page(BizCouponRecordPageParam bizCouponRecordPageParam) {
        return CommonResult.data(bizCouponRecordService.page(bizCouponRecordPageParam));
    }

    /**
     * 添加优惠券记录
     *
     * @author wulei
     * @date  2025/02/06 17:38
     */
    @Operation(summary = "添加优惠券记录")
    @CommonLog("添加优惠券记录")
    @SaCheckPermission("/biz/couponrecord/add")
    @PostMapping("/biz/couponrecord/add")
    public CommonResult<String> add(@RequestBody @Valid BizCouponRecordAddParam bizCouponRecordAddParam) {
        bizCouponRecordService.add(bizCouponRecordAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑优惠券记录
     *
     * @author wulei
     * @date  2025/02/06 17:38
     */
    @Operation(summary = "编辑优惠券记录")
    @CommonLog("编辑优惠券记录")
    @SaCheckPermission("/biz/couponrecord/edit")
    @PostMapping("/biz/couponrecord/edit")
    public CommonResult<String> edit(@RequestBody @Valid BizCouponRecordEditParam bizCouponRecordEditParam) {
        bizCouponRecordService.edit(bizCouponRecordEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除优惠券记录
     *
     * @author wulei
     * @date  2025/02/06 17:38
     */
    @Operation(summary = "删除优惠券记录")
    @CommonLog("删除优惠券记录")
    @SaCheckPermission("/biz/couponrecord/delete")
    @PostMapping("/biz/couponrecord/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   List<BizCouponRecordIdParam> bizCouponRecordIdParamList) {
        bizCouponRecordService.delete(bizCouponRecordIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取优惠券记录详情
     *
     * @author wulei
     * @date  2025/02/06 17:38
     */
    @Operation(summary = "获取优惠券记录详情")
    @SaCheckPermission("/biz/couponrecord/detail")
    @GetMapping("/biz/couponrecord/detail")
    public CommonResult<BizCouponRecord> detail(@Valid BizCouponRecordIdParam bizCouponRecordIdParam) {
        return CommonResult.data(bizCouponRecordService.detail(bizCouponRecordIdParam));
    }

    /**
     * 根据编码获取优惠券记录列表
     *
     * @author wulei
     * @date  2025/02/06 17:38
     */
    @Operation(summary = "根据编码获取优惠券记录列表")
    @SaCheckPermission("/biz/couponrecord/queryByCode")
    @GetMapping("/biz/couponrecord/queryByCode")
    public CommonResult<List<BizCouponRecord>> queryByCode(@Valid BizCouponRecordPageParam bizCouponRecordPageParam) {
        return CommonResult.data(bizCouponRecordService.queryByCode(bizCouponRecordPageParam));
    }

    /**
     * 优惠券核销
     * @param bizCouponRecordEditParam
     * @return
     */
    @Operation(summary = "优惠券核销")
    @CommonLog("优惠券核销")
    @SaCheckPermission("/biz/couponrecord/destroy")
    @PostMapping("/biz/couponrecord/destroy")
    public CommonResult<String> destroy(@RequestBody @Valid BizCouponRecordEditParam bizCouponRecordEditParam) {
        bizCouponRecordService.destroy(bizCouponRecordEditParam);
        return CommonResult.ok();
    }
}
